(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     60241,       1607]*)
(*NotebookOutlinePosition[     61038,       1634]*)
(*  CellTagsIndexPosition[     60994,       1630]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[StyleBox["Visualization of Tree Structures",
  FontSize->18,
  FontColor->RGBColor[0, 0, 0.514702]]], "Title",
  CellDingbat->None,
  Background->RGBColor[0.992645, 1, 0.647059]],

Cell[TextData[StyleBox["Examples from Genetic Programming and A Model of Tree \
Growth\n3 March 2004",
  FontSize->14]], "Subtitle"],

Cell[TextData[StyleBox["Jason M. Daida, Adam M. Hilss, David J. Ward, and \
Stephen L. Long",
  FontSize->12,
  FontSlant->"Plain",
  FontVariations->{"CompatibilityType"->0}]], "Subsubtitle"],

Cell[TextData[StyleBox["The University of Michigan\nCenter for the Study of \
Complex Systems and the Space Physics Research Laboratory\n2455 Hayward \
Avenue\nAnn Arrbor, Michigan USA 48109-2143\ndaida@eecs.umich.edu",
  FontSize->10,
  FontSlant->"Plain",
  FontVariations->{"CompatibilityType"->0}]], "Subsubtitle"],

Cell[CellGroupData[{

Cell["Abstract", "Section"],

Cell[TextData[{
  "This ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook supports two papers that were presented at GECCO 2003:\n\nDaida, \
J.M., A.M. Hilss, D.J. Ward, and S.L. Long, \
\[OpenCurlyDoubleQuote]Visualizing Tree Structures in Genetic Programming,\
\[CloseCurlyDoubleQuote]in ",
  StyleBox["Proceedings of GECCO 2003",
    FontSlant->"Italic"],
  ". (A long version of the paper appears in the ",
  StyleBox["Journal of Genetic Programming and Evolvable Machines.) ",
    FontSlant->"Italic"],
  StyleBox["This paper presents methods to visualize the structure of trees \
that occur in genetic programming. These methods allow for the inspection of \
structure of entire trees even though several thousand of nodes may be \
involved. The methods also scale to allow for the inspection of structure for \
entire populations and for complete trials even though millions of nodes may \
be involved. Examples are given that demonstrate how this new way of \
\[OpenCurlyDoubleQuote]seeing\[CloseCurlyDoubleQuote] can afford a \
potentially rich way of understanding dynamics that underpin genetic \
programming. The examples indicate further studies that might be enabled by \
visualizing structure at these scales.",
    FontFamily->"Times New Roman"],
  "\n\nDaida, J.M. and A.M. Hilss, \[OpenCurlyDoubleQuote]Identifying \
Structural Mechanisms in Genetic Programming,\[CloseCurlyDoubleQuote]in ",
  StyleBox["Proceedings of GECCO 2003",
    FontSlant->"Italic"],
  ". This paper presents a hypothesis about an undiscovered class of \
mechanisms that exist in standard GP. Rather than being intentionally \
designed, these mechanisms would be an unintended consequence of using trees \
as information structures. A model is described that predicts outcomes in GP \
that would arise solely from such mechanisms. Comparisons with empirical \
results from GP lend support to the existence of these mechanisms.\n\nThis \
notebook includes much of the Mathematica code that was used in the \
preparation of these papers. As in any Mathematica notebook, you can \
experiment with the code by executing (",
  StyleBox["shift-return",
    FontFamily->"Courier"],
  " or ",
  StyleBox["enter",
    FontFamily->"Courier"],
  ", depending on platform) the bolded mono-space font that looks like this.\n\
"
}], "Text",
  PageWidth->PaperWidth]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["Code",
  FontColor->RGBColor[0, 0, 1]]], "Section",
  InitializationCell->True],

Cell["\<\
The code here will execute when any of code in the section entitled \
\[OpenCurlyDoubleQuote]Visualization and Simulation Examples\
\[CloseCurlyDoubleQuote]is run. An alternative is to select this entire \
section and evaluate it before the examples are run.

Note that the current convention in the published papers for root node is at \
depth = 0.\
\>", "Text",
  PageWidth->PaperWidth,
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Visualization Code", "Subsection",
  InitializationCell->True],

Cell[CellGroupData[{

Cell[TextData[{
  "T",
  StyleBox["reeElementLocations[nodes] -> coordinates, graphics", 
    "Subsubsection"]
}], "Subsubsection",
  InitializationCell->True],

Cell["\<\
TreeElementLocations::usage=
\"TreeElementLocations[nodes] returns the 2D coordinates that corresponds to \
each element in the given set of nodes. Note that nodes refers to a set of \
node indices, where 1 denotes the root of the binomial tree. The indexing \
convention is starting from the root at the top to the leaves on the bottom. \
All node indices that are powers of two are on the left of the binomial tree. \
Example: For a full tree of Depth = 1, nodes = {1, 2, 3}.\"; 


TreeElementLocations[nodes_] :=

Module[{i, locations, theta, radius},
    (* -------------------------------------------------------------------*)
    (* MACROs *)
    
    (* This macro computes for possible angles at a particular depth *)
    Phi[D_, n_] :=  N[Mod[(90 + (360 / 2^D) + (n (360 / 2^(D-1)))),360] \
Degree];
    
    (* This macro computes the coordinates based on phi and depth *)
    RingElement[theta_] := {N[Cos[theta]], N[Sin[theta]]};    
        
    (* -------------------------------------------------------------------*)

\t
\t(* Allocate memory for coordinates *)
\tlocations = Table[{0,0},{Length[nodes]}];
\t\t
\t(* Convert node elements to locations *)
\tDo[
\t   radius = Ceiling[Log[2, nodes[[i]]+1]] - 1;
\t   theta  = Phi[radius+1, nodes[[i]]];
\t   locations[[i]] = radius RingElement[theta];,\t
\t{i, Length[nodes]}];
\t
 \tReturn[locations];
 \t     
]\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["GraphTreeTwo[ nodes] -> graphics ", "Subsubsection",
  InitializationCell->True],

Cell["\<\
GraphTreeTwo::usage=
\"GraphTreeTwo[nodes] generates a plot of a binary tree as a function of \
nodes and tree element locations. Note that nodes refers to a set of node \
indices, where 1 denotes the root of the binomial tree. The indexing \
convention is starting from the root at the top to the leaves on the bottom. \
All node indices that are powers of two are on the left of the binomial tree. \
Example: For a full tree of Depth = 1, nodes = {1, 2, 3}.\";


GraphTreeTwo[nodes_] :=

Module[{treeLine, evenNodes, i},
    
    (******************************************************************)
    (*  The construction of the tree starts from the leaves and works *)
    (*  down to the root. In this way, all one needs to do is to      *)
    (*  identify only the even numbered nodes.                        *)
    (******************************************************************)


\t(* identification of even nodes ----------------------------------*)
\ttreeLine = {AbsoluteThickness[0.5]};
\tevenNodes = Select[nodes, EvenQ];
\ti = Length[evenNodes];
\t
\t
\t(* iteration of even nodes ---------------------------------------*)
\tWhile[i > 0, 
 \t     treeLine = Append[treeLine,
 \t     LineElementTwo[evenNodes[[i]] ]]; i--;];
 \t     
 \tReturn[Graphics[treeLine]];
 \t     
 ]



(*====================================================================*)

LineElementTwo::usage=
\"LineElementTwo[nodeElement] LineElement does the line primitives for the \
smallest possible subtree with Depth > 1.\";


LineElementTwo[nodeElement_] :=

Module[{linePair, pointOne, pointTwo, pointThree, radius, theta},


    (* ****************************************************************)
    (* MACROs --------------------------------------------------------*)
    
    
    (* This macro computes for possible angles at a particular depth  *)
    
    Phi[D_, n_] :=  N[Mod[(90 + (360 / 2^D) + (n (360 / 2^(D-1)))),360] \t\t\t\
\t\t\tDegree];
 
    
    (* This macro computes the coordinates based on phi and depth     *)
    
    RingElement[theta_] := {N[Cos[theta]], N[Sin[theta]]};    
    
    
    (* END MACROs ----------------------------------------------------*)
    (* ****************************************************************)
    
    
    
    (* calculation for leftmost point --------------------------------*)
    radius = Ceiling[Log[2, nodeElement+1]] - 1;
\ttheta  = Phi[radius+1, nodeElement];
\tpointTwo = radius RingElement[theta];
\t
\t
\t(* calculation for rightmost point -------------------------------*)
\ttheta  = Phi[radius+1, nodeElement+1];
\tpointThree = radius RingElement[theta];\t


    (* calculation for apex point ------------------------------------*)
    radius = radius - 1;
\ttheta  = Phi[radius+1, nodeElement/2];
    pointOne = radius RingElement[theta];

\t
\t(* making lines --------------------------------------------------*)
\tlinePair = {Line[{pointOne, pointTwo}],
                Line[{pointOne, pointThree}]};

 \t     
 \tReturn[linePair];
 \t     
]

\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["GraphTreeNodesTwo[ nodes] -> graphics plot", "Subsubsection",
  InitializationCell->True],

Cell["\<\
GraphTreeNodesTwo::usage=
\"GraphTreeNodesTwo[nodes] generates a graphic of just the nodes for a \
binomial tree. Note that nodes refers to a set of node indices, where 1 \
denotes the root of the binomial tree. The indexing convention is starting \
from the root at the top to the leaves on the bottom. All node indices that \
are powers of two are on the left of the binomial tree. Example: For a full \
tree of Depth = 1, nodes = {1, 2, 3}.\";


GraphTreeNodesTwo[locations_] :=

Module[{treeNodes},    
    
\ttreeNodes = {PointSize[0.01],
\t             Thread[Point[locations]]};

 \tReturn[Graphics[treeNodes]];
 \t     
]\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["BinomialLattice[depth ] -> lattice", "Subsubsection",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["\<\
BinomialLattice::usage=
\"BinomialLattice[depth] returns a circular lattice of (x,y)  coordinates, \
which are used to plot binomial trees. Note: computation of these points can \
be fairly time consuming beyond depth = 17. The smallest possible depth for a \
non-Null lattice is 1.\";

BinomialLattice[depth_] :=

Module[{d},

(**********************************************************************)
(*  phi[] computes angle of lattice point to the x-axis. Note that    *)
(*  the starting angle is always the first angle from the y-axis.     *)      \
                                        
(**********************************************************************)
\t
\tphi[D_, n_] := N[Mod[(90+(360/ 2^D) + (n (360/ 2^(D-1)))),360] 
\tDegree];
\t
\t
(**********************************************************************)
(*  RingElement[] transforms the lattice point angle to a x,y         *)
(*  coordinate on the unit circle.                                    *)
(**********************************************************************)

\tRingElement[theta_] := {N[Cos[theta]], N[Sin[theta]]};


(**********************************************************************)
(* Ring[] computes the lattice points for a particular depth          *)
(**********************************************************************)
\t
    Ring[D_] := 
      Thread[(D - 1) *   
      RingElement[First[Array[phi,{1, 2^(D-1)}, {D, 0}]]]];
      
    
(**********************************************************************)
(* The following creates the lattice                                  *)
(**********************************************************************)

\tlattice = {{0,0}};
    Do[lattice = 
       Partition[Flatten[Append[lattice, Ring[d]]],2], 
       {d, 2, depth}];
       
       
    
    Return[lattice];
    
]

\
\>", "Input",
  InitializationCell->True],

Cell["\<\
Syntax::newl: The newline character after 
    \"\tphi[D_, n_] := N[Mod[(90+(360/ 2^D) + (n (360/ 2^(D-1)))),360] \"
     is understood as a multiplication operator.\
\>", "Message"],

Cell[BoxData[
    \(General::"spell1" \(\(:\)\(\ \)\) 
      "Possible spelling error: new symbol name \"\!\(phi\)\" is similar to \
existing symbol \"\!\(Phi\)\"."\)], "Message"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["GraphTreeNodes[ nodes, lattice] -> graphics plot", "Subsubsection",
  InitializationCell->True],

Cell["\<\
GraphTreeNodes::usage=
\"GraphTreeNodes[nodes, lattice] generates a graphic of just the nodes for a \
binomial tree. Note that nodes refers to a set of node indices, where 1 \
denotes the root of the binomial tree. The indexing convention is starting \
from the root at the top to the leaves on the bottom. All node indices that \
are powers of two are on the left of the binomial tree. Example: For a full \
tree of Depth = 1, nodes = {1, 2, 3}. Assume that lattice is one that is \
generated from BinomialLattice[], although any lattice with the same indexing \
scheme can be used.\";


GraphTreeNodes[nodes_, lattice_] :=

Module[{treeNodes},    

    (* error checking ------------------------------------------------*)
    If[Max[nodes] > Length[lattice],
       Print[\"PlotTree Error: Lattice is too small to show all nodes.
             Try using a larger lattice (i.e., use BinomialLattice with 
             a larger value of Depth.\"];
   
       Return[];

      ];    
    
\t
\t(* generation of graphics ----------------------------------------*)
\ttreeNodes = {PointSize[0.01],
\t             Thread[Point[lattice[[nodes]]]]};

 \tReturn[Graphics[treeNodes]];
 \t     
]\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["IdentifyLeafs[ nodes] -> leaf nodes only", "Subsubsection",
  InitializationCell->True],

Cell["\<\
IdentifyLeafs::usage=
\"IdentifyLeafs[nodes] returns leaf nodes that exist in nodes.\";


IdentifyLeafs[nodes_] :=

Module[{maxDepth, depth, leafs, leafsTmp, evenLeafs,
        innerNodes,remainingNodes},
   
    (* This algorithm searches for leafs by depth --------------------*)
\tmaxDepth = Ceiling[Log[2, Max[nodes]+1]];


\t(* check for single-node tree ------------------------------------*)
\tIf[maxDepth == 1, Return[nodes]];
\t

\t(* calculations --------------------------------------------------*)
\tleafs = Select[nodes, #1 >= 2^(maxDepth-1)&];
\t

\t(* check for unwed leafs -----------------------------------------*)
\tIf[ Not[EvenQ[Length[leafs]]],
\t\t    Print[\"IdentifyLeaf Warning: The tree has an odd number of \t\t\t\t\t\
\t\tnodes at depth = \",maxDepth];
\t  ];
\t
\t
\t(* calculations continued ----------------------------------------*)
\tevenLeafs = Select[leafs, EvenQ];
\tinnerNodes = evenLeafs/2;
\tremainingNodes = Complement[nodes, leafs];
\t
\tIf[maxDepth == 2, Return[leafs]];


\t(* iteration for trees of depth greater than two -----------------*)\t
\tFor[depth=maxDepth-1, depth > 1, depth--,

        (**************************************************************)
\t    (*  note that the ordering is somewhat different from the     *)
\t    (*  case depth = 2                                            *)
\t    (**************************************************************)
\t
\t
\t\tleafsTmp = Select[remainingNodes, #1 >= 2^(depth-1)&];


\t\t(* check for null layers -------------------------------------*)
\t\tIf[leafsTmp == {},
\t\t   Print[\"IdentifyLeaf Warning: The tree has a null layer at
\t\t   depth = \",depth];
\t\t  ];
\t\t
\t\t
\t\t(* check for unwed nodes -------------------------------------*)
\t\tIf[ Not[EvenQ[Length[leafsTmp]]],
\t\t    Print[\"IdentifyLeaf Warning: The tree has an odd number of \t\t\t\t\t\
\t\tnodes at depth = \",depth];
\t\t  ];
\t
\t
\t\t(* calculations ----------------------------------------------*)
\t\tevenLeafs = Select[leafsTmp, EvenQ];\t\t
\t\tremainingNodes = Complement[remainingNodes, leafsTmp];
\t\tleafsTmp = Complement[leafsTmp,innerNodes];
\t\tleafs = Union[leafs, leafsTmp];
\t\t
\t\t
\t\tinnerNodes = evenLeafs/2;
\t
\t   ];


\tReturn[leafs];\t
 \t     
]\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["NodesToRGTform[ nodes] -> nodesRGT", "Subsubsection",
  InitializationCell->True],

Cell["\<\
NodesToRGTform::usage=
\"NodesToRGTform[nodes] returns the list of nodes in a format that \
RandomGrowTree[] can use.The format is basically in a two element list, where \
the first element is a list of internal nodes, while the second element is a \
list of leafs. Note that nodes are assumed to be ordered from lowest to \
highest, starting from 1.\";


NodesToRGTform[nodes_] :=

Module[{leafs, internalNodes, rgtFormattedNodes},
\tleafs = IdentifyLeafs[nodes];
\tinternalNodes = Complement[nodes, leafs];
\trgtFormattedNodes = {internalNodes, leafs};
\tReturn[rgtFormattedNodes];     
]\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["RGTtoNodesForm[nodesRGT ] -> nodes", "Subsubsection",
  InitializationCell->True],

Cell["\<\
RGTtoNodesForm::usage=
\"RGTtoNodesForm[nodesRGT] returns the list of nodes in a format that \
PlotTree[] can use.\";


RGTtoNodesForm[nodesRGT_] := Sort[Flatten[nodesRGT]]
\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Routine for generating node summaries for a population \
(SummarizePopulation)\
\>", "Subsubsection",
  InitializationCell->True],

Cell["\<\
This routine reads in the .nod files from Hilss and prepares them \
for use with GrayscaleTrees. Note that each nod file corresponds to one tree.\
\
\>", "Text",
  PageWidth->PaperWidth,
  InitializationCell->True],

Cell[BoxData[
    \(\(\(\[IndentingNewLine]\)\(\(SummarizePopulation::usage = \ \ \
"\<SummarizePopulation[dirPathName,numberOfTrials]returns a population \
summary in the following format: {{node label a, total number of trees that \
have node label a}, {node label b, total number of trees that have node label \
b},...}. Note that this form is compatible with the key form of trees (which \
is used to generate color trees).\>";\)\[IndentingNewLine]\n
    SummarizePopulation[dirPathName_, 
        numberOfTrials_] := \[IndentingNewLine]\ \ \ \ \ \ \ \ \
\[IndentingNewLine]\ \ \ \ \ \ Module[{i, j, dataIndividual, \ dataPop, 
          treeDir, fileRootName, \ \[IndentingNewLine]fileSuffix, filename, 
          sortedDataPop, \ elementsDataPop, \ totalLabels, labelCount, \ 
          popSummaryByNodeLabel}, \[IndentingNewLine]\[IndentingNewLine] (*\(\
**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)*******************) \[IndentingNewLine] (*\ \ These\ \
variables\ are\ hardcoded\ and\ may\ need\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ *) \ \ \ \  (*to\ be\ changed\ to\ suit\ other\ parsed\ data\ sets\ \ \
\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ *) \[IndentingNewLine]\
\[IndentingNewLine]fileRootName\  = \ "\<tree\>"; \
\[IndentingNewLine]fileSuffix\  = \ "\<.nod\>"; \[IndentingNewLine] \
(*****************************************************************************\
*) \[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]treeDir\  = \ 
          dirPathName; \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine] (*\ 
          Read\ in\ and\ concatenate\ \
\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(data\ --\)--\)--\)--\)--\)--\)--\)--\)\
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\) - \(\(*\)\(--\)\)\
*) \[IndentingNewLine]For[i\  = \ 1, \ 
          i \[LessEqual] 
            numberOfTrials, \ \(i++\), \
\[IndentingNewLine]{\[IndentingNewLine]filename\  = \ 
              treeDir <> fileRootName <> ToString[i - 1] <> 
                fileSuffix; \[IndentingNewLine]dataIndividual\  = \ 
              ReadList[filename, \ 
                RecordSeparators \[Rule] {"\<\n\>"}]; \[IndentingNewLine]If[
              i\  \[NotEqual] \ 
                1, \[IndentingNewLine]\(dataPop\  = \ 
                  Join[dataPop, \ 
                    dataIndividual];\), \[IndentingNewLine]\(dataPop\  = \ 
                  dataIndividual;\)];\[IndentingNewLine]}]; \
\[IndentingNewLine]sortedDataPop\  = \ 
          Sort[dataPop]; \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine] (*\ \(Prepare\ for\ \
\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(counting\ --\)--\)--\)--\)\
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\
--\)--\)\)\(-\)*) \[IndentingNewLine]elementsDataPop\  = \ 
          Union[sortedDataPop]; \[IndentingNewLine]totalLabels\  = \ 
          Length[elementsDataPop]; \[IndentingNewLine]popSummaryByNodeLabel\  \
= \ Table[{0, 0}, {i, \ 
              totalLabels}]; \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine] (*\ \(Count\ labels\ as\ they\ appear\ in\ a\ \
\(\(\(\(\(\(\(\(\(\(\(\(population\ --\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\
--\)--\)\)\(-\)*) \[IndentingNewLine]j\  = \ 1; \[IndentingNewLine]For[
          i\  = \ 1, \ 
          i \[LessEqual] 
            totalLabels, \ \(i++\), \[IndentingNewLine]\[IndentingNewLine]\t
          labelCount\  = \ 0; \[IndentingNewLine]\t
          While[elementsDataPop[\([i]\)]\  \[Equal] \ 
              sortedDataPop[\([j]\)], \[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \
\ \[IndentingNewLine]\t\(labelCount++\); \[IndentingNewLine]\t\(j++\); \
\[IndentingNewLine]\t
            If[j\  > \ Length[sortedDataPop], Break[], 
              Continue[]];\[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \
\[IndentingNewLine]\ \ \ \ \ \ \ ]; \[IndentingNewLine]\t
          popSummaryByNodeLabel[\([i]\)]\  = {elementsDataPop[\([i]\)], \ 
              labelCount};\[IndentingNewLine]\[IndentingNewLine]\t\
\[IndentingNewLine]]; \[IndentingNewLine]\[IndentingNewLine]Return[
          popSummaryByNodeLabel];\[IndentingNewLine]\[IndentingNewLine]]\
\[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]
    \)\)\)], "Input",
  PageWidth->WindowWidth,
  CellMargins->{{Inherited, 0}, {Inherited, Inherited}},
  InitializationCell->True,
  CellSize->{562, Inherited},
  TextAlignment->Left,
  TextJustification->0]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Hilss\[CloseCurlyQuote]s method for visualizing populations \
(GrayscaleTree)\
\>", "Subsubsection",
  InitializationCell->True],

Cell[BoxData[{
    \(\(\(LineElementOne::usage = "\<LineElementOne[nodesElement] returns a \
line graphic element that is used to build population-level visualization of \
binary trees. This function is treated as a subroutine to GrayscaleTree. \
(Authored by Hilss January 2002).\>";\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(LineElementOne[nodeElement_] := 
      Module[{linePair, pointOne, pointTwo, radius, 
          theta}, \[IndentingNewLine]\[IndentingNewLine] (*\ \(**\)\(**\)\(\
**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)***************) \[IndentingNewLine] (*\ \(\(\(\(\(\(\(\(\(\(\(\(\(\
\(\(\(\(\(\(\(\(\(\(MACROs\ --\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\) - \(--\(--\(--\(--\(--\(--\(--\(\
--\(----\)\)\)\)\)\)\)\)\)*) \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine] (*This\ macro\ computes\ for\ possible\ angles\ at\ a\ \
particular\ depth\ \ \ \ \ *) \[IndentingNewLine]\t
        Phi[D_, n_] := 
          N[Mod[\((90 + \((360/2^D)\) + \((n\ \((360/2^\((D - 1)\))\))\))\), 
                360]\ Degree]; \[IndentingNewLine]\[IndentingNewLine] (*This\ \
macro\ computes\ the\ coordinates\ based\ on\ phi\ and\ depth\ \ \ \ \ \ \ \ \
*) \[IndentingNewLine]\t
        RingElement[theta_] := {N[Cos[theta]], 
            N[Sin[theta]]}; \ \ \ \ \[IndentingNewLine]\[IndentingNewLine] \
(*\ \(END\ \(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(MACROs\
\ --\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\)\(-\)*) \[IndentingNewLine] (*\ \
 ** *************************************************************************)\
 \[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine] \
(*\ calculations\ for\ leftmost\ \(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(point\ \
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\
*) \[IndentingNewLine]radius = 
          Ceiling[Log[2, nodeElement[\([1]\)] + 1]] - 
            1; \[IndentingNewLine]theta = 
          Phi[radius + 1, 
            nodeElement[\([1]\)]]; \[IndentingNewLine]pointTwo = 
          radius\ RingElement[
              theta]; \[IndentingNewLine]\t\[IndentingNewLine] (*\ 
          calculations\ for\ apex\ \
\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(point\ --\)--\)--\)--\)--\)--\)--\)\
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)*) \[IndentingNewLine]\
radius = radius - 1; \[IndentingNewLine]theta = 
          Phi[radius + 1, 
            Floor[nodeElement[\([1]\)]/2]]; \[IndentingNewLine]pointOne = 
          radius\ RingElement[
              theta]; \[IndentingNewLine]\[IndentingNewLine] (*\ 
          creation\ of\ \
\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(lines\ --\)--\)--\)--\)\
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\
--\)--\)--\)--\)*) \[IndentingNewLine]linePair\  = \ 
          Line[{pointOne, pointTwo}]; \[IndentingNewLine]Return[
          linePair];]\), "\[IndentingNewLine]", 
    \(\)}], "Input",
  InitializationCell->True],

Cell[BoxData[{
    \(\(\(GrayscaleTree::usage = "\<GrayscaleTree[populationLabelSummary] \
returns a graphic element that is a population-level visualization of binary \
trees (i.e., do Show[GrayscaleTree[populationLabelSummary]] to see). The \
variable populationLabelSummary is 2 x N array. The first column are the node \
labels (type integer), which uses our labeling conventions. The second column \
is the number of times in a population that each node label has appeared \
(also of type integer). (Authored by Hilss January 2002; minor bug fix Daida \
January 2003).\>";\)\(\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\(\(GrayscaleTree[
          data_]\  := \[IndentingNewLine]Module[{i, x, \ totalTrees, 
            newData, \ treeLine}, \[IndentingNewLine]\t
          newData = 
            Array[0, {Length[data], 
                2}]; \[IndentingNewLine]\[IndentingNewLine]\t
          totalTrees = 
            data[\([1, 
                2]\)]; \[IndentingNewLine]\[IndentingNewLine]\
\[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \  (*\ 
            iteration\ for\ establishment\ of\ color\ \
\(\(\(\(\(\(\(\(\(scheme\ --\)--\)--\)--\)--\)--\)--\)--\)--\)\
*) \[IndentingNewLine]\t
          For[i = 1, \ 
            i \[LessEqual] 
              Length[data], \ \(i++\), \[IndentingNewLine]\t\
\[IndentingNewLine]\t\tx = 
              data[\([i]\)]; \[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \[IndentingNewLine]\[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ \ \ \ \ \ \ \  (*\ 
              Set\ \(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(\(colors\ \
--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)--\)\
--\)--\)--\)--\)--\)--\)*) \[IndentingNewLine]\t\tIf[
              x[\([2]\)]/totalTrees\  < \  .01, \ 
              x[\([2]\)] = 
                RGBColor[ .9,  .9,  .9], \t\ \ \ \ \[IndentingNewLine]\t\t\t\
\(If[x[\([2]\)]/
                      totalTrees\  < \  .05, \ \[IndentingNewLine]\t\t\t\
x[\([2]\)] = RGBColor[ .8,  .8,  .8], \t\ \ \ \ \[IndentingNewLine]\t\t\t\(If[
                      x[\([2]\)]/
                          totalTrees\  < \  .1, \ \ \[IndentingNewLine]\t\t\t\
x[\([2]\)] = 
                        RGBColor[ .6,  .6,  .6], \ \ \ \ \ \ \ \
\[IndentingNewLine]\t\t\t\(If[
                          x[\([2]\)]/
                              totalTrees\  < \  .2, \ \[IndentingNewLine]\t\t\
\t\ \ \ x[\([2]\)] = RGBColor[ .4,  .4,  .4], \[IndentingNewLine]\t\t\t\t\(If[
                              x[\([2]\)]/totalTrees\  < \  .5, \ 
                              x[\([2]\)] = \[IndentingNewLine]\t\t\t\t\
RGBColor[ .2,  .2,  .2], \ \ \ \ \ \ \ \ \ \ \ \[IndentingNewLine]\t\t\t\t\t\
\(x[\([2]\)] = RGBColor[0, 0, 
                                    0];\)\[IndentingNewLine]\t\t\t\ \ \ \ \ \ \
\ \ \ \ \ ];\)\[IndentingNewLine]\t\t\t\ \ ];\)\[IndentingNewLine]\t\t\ \ \ \ \
\ \ \ \ \ \ \ \ ];\)\[IndentingNewLine]\t\t\ \ \ \ \ \ \ \ \ \ \ ];\)\
\[IndentingNewLine]\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ ]; \
\[IndentingNewLine]\t\[IndentingNewLine]\t\tnewData[\([i]\)] = 
              x;\[IndentingNewLine]\t]; \
\[IndentingNewLine]\[IndentingNewLine]\[IndentingNewLine]\t
          treeLine = {AbsoluteThickness[0.5], 
              RGBColor[0, 0, 1]}; \[IndentingNewLine]\[IndentingNewLine]\t
          i = Length[
              newData]; \[IndentingNewLine]\[IndentingNewLine]\ \ \ \ \ \
\[IndentingNewLine]\t
          While[i > 
              1, \[IndentingNewLine]\t\t\({treeLine = 
                Append[treeLine, 
                  newData[\([i, 2]\)]]; \[IndentingNewLine]\t\ \ \ treeLine = 
                Append[treeLine, 
                  LineElementOne[newData[\([i]\)]]];\ }\[IndentingNewLine]
            \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \(\(i--\);\)\)]; \
\[IndentingNewLine]\[IndentingNewLine]\t
          Return[Graphics[
              treeLine]];\[IndentingNewLine]\[IndentingNewLine]];\)\(\
\[IndentingNewLine]\)
    \)\), "\[IndentingNewLine]", 
    \(\)}], "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Routine for visualizing a complete trial (Grayscale Movie)", \
"Subsubsection"],

Cell["\<\
This code presumes the existence of a text file that summarizes the \
population for every generation of a run. These text files are called Stat \
files, a number of which are found in the GPEM/ directory.\
\>", "Text",
  PageWidth->PaperWidth],

Cell[BoxData[
    \(\(\(\[IndentingNewLine]\)\(\(VisualizeTrialGray::usage = \
"\<VisualizeTrialGray[dirPathName,numberOfGenerations, maxDepth] creates a \
movie that summarizes the dynamics of structure within a given trial. It \
presumes the following: that data is in a text file prefixed by 'stat'; that \
each 'stat' file is can be read into the following format: {{node label a, \
total number of trees that have node label a}, {node label b, total number of \
trees that have node label b},...}; that each 'stat' file corresponds to the \
population of one generation; that each generation corresponds to one frame \
of this movie; that the maximum number of trees for an entire trial is the \
number of trees at the root node at generation 1; that there are no more than \
999 generations to be visualized. The variable dirPathName is the string that \
describes where the stat files are. The variable numberOfGenerations is \
equivalent to the number of stat files to be processed. The variable maxDepth \
is the maximum depth that is to be visualized in this movie. It uses \
GrayscaleTree for visualization.\>";\)\[IndentingNewLine]\n
    VisualizeTrialGray[dirPathName_, numberOfGenerations_, \ 
        maxDepth_] := \[IndentingNewLine]Module[{j, \ dataPop, statDir, 
          fileRootName, fileSuffix, 
          filename, \[IndentingNewLine]circleGraph, \ 
          tree0}, \[IndentingNewLine] \
(*\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)\(**\)*******************) \[IndentingNewLine] (*\ 
          These\ variables\ are\ hard - 
            coded\ and\ may\ \ *) \[IndentingNewLine] (*\ 
          need\ to\ be\ changed\ to\ suit\ other\ parsed\ \
*) \[IndentingNewLine] (*\ 
          data\ sets\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \
\ \ *) \[IndentingNewLine]fileRootName\  = \ "\<stat\>"; \
\[IndentingNewLine]fileSuffix\  = \ "\<.nod\>"; \[IndentingNewLine] \
(*\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(**\)\(\
**\)\(**\)\(**\)\(**\)*******************) \[IndentingNewLine]\
\[IndentingNewLine]statDir\  = \ 
          dirPathName; \[IndentingNewLine]\[IndentingNewLine]circleGraph = 
          Graphics[{GrayLevel[0.5], 
              Circle[{0, 0}, \ 
                maxDepth]}]; \[IndentingNewLine]\[IndentingNewLine]For[
          j\  = \ 1, \ 
          j\  \[LessEqual] \ 
            numberOfGenerations, \ \(j++\), \[IndentingNewLine]\(If[
              j\  < \ 10, \ \[IndentingNewLine]\t
              nodes\  = 
                ReadList[
                  statDir <> fileRootName <> "\<00\>" <> ToString[j] <> 
                    fileSuffix, \ \[IndentingNewLine]\t\tNumber, \ 
                  RecordSeparators \[Rule] {"\<\n\>"}, 
                  RecordLists \[Rule] True]; \[IndentingNewLine]\t
              tree0\  = 
                Show[GrayscaleTree[nodes], \ 
                  circleGraph, \[IndentingNewLine]\t\t\ AspectRatio \[Rule] 
                    1, PlotRange \[Rule] All];, \[IndentingNewLine]\t\(If[
                  j\  < \ 100, \[IndentingNewLine]\t\tnodes\  = 
                    ReadList[
                      statDir <> fileRootName <> "\<0\>" <> ToString[j] <> 
                        fileSuffix, \[IndentingNewLine]\t\t\ Number, \ 
                      RecordSeparators \[Rule] {"\<\n\>"}, 
                      RecordLists \[Rule] 
                        True]; \[IndentingNewLine]\t\ttree0\  = 
                    Show[GrayscaleTree[nodes], \ 
                      circleGraph, \ \[IndentingNewLine]\t\tAspectRatio \
\[Rule] 1, PlotRange \[Rule] All];, \[IndentingNewLine]\t\t\(If[
                      j\  < \ 1000, \[IndentingNewLine]\t\t\tnodes\  = 
                        ReadList[
                          statDir <> fileRootName <> ToString[j] <> 
                            fileSuffix, \[IndentingNewLine]\t\t\t\ Number, \ 
                          RecordSeparators \[Rule] {"\<\n\>"}, 
                          RecordLists \[Rule] 
                            True]; \[IndentingNewLine]\t\t\ttree0\  = 
                        Show[GrayscaleTree[nodes], \ 
                          circleGraph, \ \[IndentingNewLine]\t\t\tAspectRatio \
\[Rule] 1, PlotRange \[Rule] 
                            All];, \[IndentingNewLine]\t\t\t\(Print["\<Need \
to fix this if more than 999 generations.\n\>"];\)\[IndentingNewLine]\t\t];\)\
\[IndentingNewLine]\t];\)\[IndentingNewLine]];\)\[IndentingNewLine]]\
\[IndentingNewLine]]\)\)\)], "Input",
  InitializationCell->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Simulation Code", "Subsection",
  InitializationCell->True],

Cell["\<\
The code here is only a portion of the code that was used in the \
GECCO papers. This particular version is suited to visually demonstrate \
principles in \[OpenCurlyDoubleQuote]Identifying Structural Mechanism in \
Genetic Programming.\[CloseCurlyDoubleQuote]  The actual code is embedded in \
an Monte Carlo routine that  statistically characterizes the model.  It \
requires a fair amount of time to generate a map like the one that was given \
in \[OpenCurlyDoubleQuote]Identifying Structural Mechanisms in Genetic \
Programming\[CloseCurlyDoubleQuote] and \[OpenCurlyDoubleQuote]What Makes a \
Problem GP-Hard?\[CloseCurlyDoubleQuote] Our particular map took us about one \
CPU-year to generate.\
\>", "Text",
  PageWidth->PaperWidth],

Cell[CellGroupData[{

Cell["LatticeAggregate3, 3-node growth only Code", "Subsubsection",
  InitializationCell->True],

Cell["\<\
LatticeAggregate3::usage=
\"LatticeAggregate3[targetDepth, seed] generates a random binomial tree that \
starts from a three node seed up until the proximity of targetDepth. Variable \
seed refers to an integer that seeds the random number generator. This \
variant uses only 3-node growth.\";



LatticeAggregate3[targetDepth_?IntegerQ, seed_?IntegerQ] :=

Module[{seedTree,nodesRGT,internalNodes,leafs,initialDepth,
        currentDepth, iteration, numberLeafs, selectLeafIndex,
        newLeafs,newInternalNode},
    
    SeedRandom[seed];

    seedTree = {1, 2, 3};
    nodesRGT = NodesToRGTform[seedTree];
           
        
    internalNodes = nodesRGT[[1]];
\tleafs = nodesRGT[[2]];
\tinitialDepth=Ceiling[Log[2, Max[leafs]+1]];
\tcurrentDepth=initialDepth;
\t
\t
\t(* check for invalid trees ---------------------------------------*)
\tIf[leafs == {},
\t\tPrint[\"RandomGrowTree Warning: The initial tree has no leafs.\"];
\t\tReturn[nodesRGT];
\t];
\t\t
\t
\titeration = 0;
\t\t

\t(******************************************************************)
\t(*  Note that the following termination criteria means that trees *)
\t(*  will grow until the first time the tree reaches or exceeds    *)
\t(*  maximumDepth.  In this case, the maximumIterations feature    *)
\t(*  has been disabled.                                            *)
\t(******************************************************************)
\t
\t
\tWhile[currentDepth < targetDepth,
\t    
\t\tnumberLeafs = Length[leafs];
\t\tselectLeafIndex = Random[Integer, {1, numberLeafs}];
\t\t
\t\tnewLeafs = {2 leafs[[selectLeafIndex]],
\t\t        \t2 leafs[[selectLeafIndex]] + 1};
\t\t            
\t    newInternalNode = {leafs[[selectLeafIndex]]};
\t\t
\t    internalNodes = 
\t\t\t  Union[internalNodes, newInternalNode];
\t\tleafs = Complement[leafs, newInternalNode];
\t\tleafs = Union[leafs, newLeafs];
\t\tcurrentDepth = Ceiling[Log[2, Max[leafs]+1]];
\t\t
\t\titeration++;
\t\t
\t];
\t
\t
\t(* The output here is in RGT form --------------------------------*)
\tReturn[{internalNodes, leafs}];   
]\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["LatticeAggregate3Movie, 3-node growth only Code", "Subsubsection",
  InitializationCell->True],

Cell["\<\
LatticeAggregate3Movie::usage=
\"LatticeAggregate3Movie[targetDepth, iterationsPerFrame, seed] generates a \
random binomial tree that starts from a three node seed up until the \
proximity of targetDepth. The parameter iterationsPerFrame determines how \
many iterations there are for each visualization. This variant uses only \
3-node growth.This particular code generates a movie of simulated tree \
growth.\";



LatticeAggregate3Movie[targetDepth_?IntegerQ, iterationsPerFrame_?IntegerQ, \
seed_?IntegerQ] :=

Module[{seedTree, nodesRGT, internalNodes, leafs, initialDepth,
       currentDepth, iteration, circleGraph, numberLeafs,
       selectLeafIndex, newLeafs, newInternalNode},
    
    SeedRandom[seed];

    seedTree = {1, 2, 3};
    nodesRGT = NodesToRGTform[seedTree];
           
        
    internalNodes = nodesRGT[[1]];
\tleafs = nodesRGT[[2]];
\tinitialDepth=Ceiling[Log[2, Max[leafs]+1]];
\tcurrentDepth=initialDepth;
\t
\t(* check for invalid trees ---------------------------------------*)
\tIf[leafs == {},
\t\tPrint[\"RandomGrowTree Warning: The initial tree has no leafs.\"];
\t\tReturn[nodesRGT];
\t];
\t\t
\t
\titeration = 0;
\t\t
    (* Code for graphics ---------------------------------------------*)
    circleGraph = Graphics[{GrayLevel[0.5],Circle[{0, 0}, 
                  targetDepth]} ];
    Show[GraphTreeTwo[RGTtoNodesForm[nodesRGT]], 
\t\t     circleGraph,  AspectRatio->1, PlotRange -> All];
    
\t
\t(******************************************************************)
\t(*  Note that the following termination criteria means that trees *)
\t(*  will grow until the first time the tree reaches or exceeds    *)
\t(*  maximumDepth.  In this case, the maximumIterations feature    *)
\t(*  has been disabled.                                            *)
\t(******************************************************************)
\t

\tWhile[currentDepth < targetDepth,
\t    
\t\tnumberLeafs = Length[leafs];
\t\tselectLeafIndex = Random[Integer, {1, numberLeafs}];
\t\t
\t\tnewLeafs = {2 leafs[[selectLeafIndex]],
\t\t        \t2 leafs[[selectLeafIndex]] + 1};
\t\t            
\t    newInternalNode = {leafs[[selectLeafIndex]]};
\t\t
\t    internalNodes = 
\t\t\t  Union[internalNodes, newInternalNode];
\t\tleafs = Complement[leafs, newInternalNode];
\t\tleafs = Union[leafs, newLeafs];
\t\tcurrentDepth = Ceiling[Log[2, Max[leafs]+1]];
\t\t
\t\t
\t\tIf[Mod[iteration, iterationsPerFrame]==0,
\t\t\tShow[GraphTreeTwo[RGTtoNodesForm[{internalNodes, leafs}]], 
\t\t     \t circleGraph,  AspectRatio->1, PlotRange -> All];
\t\t];
\t\t     
\t\t     
\t\titeration++;
\t\t
\t\t
\t\t
\t];
\t
\t
\t(* Print[currentDepth,\" \", iteration]; ---------------------------*)
\tReturn[{internalNodes, leafs}];   
]\
\>", "Input",
  InitializationCell->True]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Other", "Subsection",
  InitializationCell->True],

Cell[CellGroupData[{

Cell["Additional Packages", "Subsubsection",
  InitializationCell->True],

Cell["\<\
<<Statistics`DescriptiveStatistics`
<< Graphics`Graphics`\
\>", "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Maximum Number of Allowable Nodes", "Subsubsection",
  InitializationCell->True],

Cell["\<\
MaximumLattice::usage=
\"MaximumLattice[depth] returns the maximum number of occupied lattice sites \
for a given depth.\";


MaximumLattice[depth_] := 2^depth - 1;
\
\>", "Input",
  InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Visualization and Simulation Examples", "Section"],

Cell[CellGroupData[{

Cell["Visualization of a Complete Plane Binary Tree", "Subsection"],

Cell[TextData[{
  "In the two GECCO papers, tree structures are represented by a set of \
positive integers. This can be demonstrated by the following code, which \
generates a set of positive integers that is bijective to a complete plane \
binary tree. The variable ",
  StyleBox["depthFullTree",
    FontFamily->"Courier"],
  " specifies the size of the tree to be generated."
}], "Text",
  PageWidth->PaperWidth],

Cell["\<\
Notes: (1) It might not be such a good idea to generate full trees \
of depth greater than 10, since the amount of memory that is needed goes up \
as an exponent of depth. (2) The convention is depth = 0 for the root node.\
\
\>", "Text",
  PageWidth->PaperWidth],

Cell[BoxData[
    \(\(depthFullTree = 8;\)\)], "Input"],

Cell[TextData[{
  "The following line of code generates the set of positive integers that is \
bijective with a complete plane binary tree of ",
  StyleBox["depthFullTree",
    FontFamily->"Courier"],
  ". "
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[
    \(\(nodes = Table[x, {x, 2^\((depthFullTree + 1)\) - 1}];\)\)], "Input"],

Cell["\<\
The next several lines of code visualize this set of positive \
integers as a tree.\
\>", "Text"],

Cell[BoxData[{
    \(\(lattice = BinomialLattice[depthFullTree + 1];\)\), "\n", 
    \(\(completeTree = GraphTreeTwo[nodes];\)\), "\[IndentingNewLine]", 
    \(Show[completeTree, AspectRatio \[Rule] 1]\)}], "Input"],

Cell[TextData[{
  "Note: While it is possible to visualize an arbitrary set of positive \
integers as a tree, the visualization code isn\[CloseCurlyQuote]t set up for \
it. The visualization code ",
  StyleBox["assumes",
    FontSlant->"Italic"],
  " that a set of positive integers is bijective to ",
  StyleBox["one",
    FontSlant->"Italic"],
  " plane binary tree, in which the root node is at the center of the \
visualization. This assumption simplifies the code and speeds up the \
visualization, but at the cost of not being able to visualize arbitrary \
integer sets."
}], "Text",
  PageWidth->PaperWidth]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualization of an Arbitrary-Sized Simulated Tree", "Subsection"],

Cell[TextData[{
  "This section illustrates the use of the lattice-aggregate model ",
  StyleBox["LatticeAggregate3",
    FontFamily->"Courier"],
  " , as well as vertex and edge representations for visualizing the results \
from this model. The variables for the model are listed here as target_depth \
and random_seed. (Note: The convention is that depth = 0 at the root node.) \
Change these two variables here to experiment with the model."
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[{
    \(\(targetDepth\  = \ 20;\)\), "\[IndentingNewLine]", 
    \(\(randomSeed\  = \ 4;\)\)}], "Input"],

Cell["This line of code executes the model:", "Text"],

Cell["testRGT = LatticeAggregate3[targetDepth,randomSeed];", "Input"],

Cell["\<\
Here is the tree using an edge representation (which was used in \
the GECCO visualization paper. A reference circle is also provided to keep \
the graphic in proper proportion.\
\>", "Text",
  PageWidth->PaperWidth],

Cell["\<\
testNodes=RGTtoNodesForm[testRGT];
testTree = GraphTreeTwo[testNodes];
circleGraph=Graphics[{GrayLevel[0.5],Circle[{0,0},targetDepth]}];
Show[testTree, circleGraph,  AspectRatio->1, PlotRange -> All]\
\>", "Input"],

Cell["\<\
Here is an alternative, vertex representation of the tree (i.e., \
plots only the nodes and not the branches). \
\>", "Text",
  PageWidth->PaperWidth],

Cell[BoxData[{
    \(\(testNodes = RGTtoNodesForm[testRGT];\)\), "\n", 
    \(\(treeCoordinates = 
        TreeElementLocations[testNodes];\)\), "\[IndentingNewLine]", 
    \(\(circleGraph = 
        Graphics[{GrayLevel[0.5], 
            Circle[{0, 0}, targetDepth]}];\)\), "\[IndentingNewLine]", 
    \(\(verticesOnly\  = \ 
        GraphTreeNodesTwo[treeCoordinates];\)\), "\[IndentingNewLine]", 
    \(Show[verticesOnly, circleGraph, AspectRatio \[Rule] 1, 
      PlotRange \[Rule] All]\)}], "Input"],

Cell["\<\
Both edge and vertex representations can be combined to form the \
following graphic. For this particular example, the reference circle has been \
removed just as an illustration of the possible distortion that can occur \
without it.\
\>", "Text",
  PageWidth->PaperWidth],

Cell[BoxData[
    \(Show[testTree, verticesOnly, AspectRatio \[Rule] 1]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simulated Growth by Three-Node Subtree", "Subsection"],

Cell[TextData[{
  "The code ",
  StyleBox["LatticeAggregate3Movie",
    FontFamily->"Courier"],
  " generates an animation of random tree growth by three-node subtrees. The \
following variables determine the particulars of this animation. The variable \
",
  StyleBox["targetDepth",
    FontFamily->"Courier"],
  " establishes the stop criterion of the movie: when a tree grows to reach \
the target depth, the animation will stop. The variable ",
  StyleBox["iterationsPerFrame",
    FontFamily->"Courier"],
  " establishes how many subtrees are attached to the tree before one \
visualization is generated. While the minimum number of iterationsPerFrame is \
one, such animations can get tedious. The variable ",
  StyleBox["randomSeed",
    FontFamily->"Courier"],
  " initializes the process: a different random seed should produce a \
different tree."
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[{
    \(\(targetDepth = 20;\)\), "\[IndentingNewLine]", 
    \(\(iterationsPerFrame\  = \ 10;\)\), "\[IndentingNewLine]", 
    \(\(randomSeed\  = \ 10;\)\)}], "Input"],

Cell[BoxData[
    \(\(LatticeAggregate3Movie[targetDepth, iterationsPerFrame, 
        randomSeed];\)\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualization of GP-Generated Tree Structures", "Subsection"],

Cell[TextData[{
  "The visualization of an individual program that has been generated by \
Genetic Programming does require a method of labelling nodes. This method was \
described in \"Visualizing Tree Structures in Genetic Programming,\" in GECCO \
2003.\n\n",
  StyleBox["We do not provide this software.",
    FontSlant->"Italic"],
  " The lex/yacc parser that was coded was specific to a particular GP kernel \
for a particular problem. It is not particularly generalizable. The labeling \
method, however, follows a pre-order traversal, which in and of itself is not \
difficult to code. If you do decide to code this method, we do not \
recommended that this method be built into a kernel.   It requires a unique \
integer label attached to each node, so it does not take much by way of tree \
depth to force the creation of labels that exceeds machine precision.\n\n\
Instead, we provide two sets of examples that have already been parsed. Each \
set corresponds to a population of a single GP run. Each set contains 500 \
files, where each file contains the node labels of a single tree. Both sets \
are from the binomial-3 (See J. Daida et al., \"What Makes a Problem GP-Hard? \
Analysis of a Tunably Difficult Problem in Genetic Programming,\" ",
  StyleBox["Journal of Genetic Programming and Evolvable Machines",
    FontSlant->"Italic"],
  ", (",
  StyleBox["2",
    FontWeight->"Bold"],
  "):2, pp. 165\[Dash]191.) The files in directory ",
  StyleBox["1C/",
    FontFamily->"Courier"],
  " correspond to the population of GP individuals that was taken at the \
200th generation for alpha = 1.The files in directory ",
  StyleBox["1kC/",
    FontFamily->"Courier"],
  " correspond to the population of GP individuals that was taken at the \
200th generation for alpha = 1000. The run parameters were otherwise the same \
as used in the GPEM paper.\n\nTo experiment with this code, a user needs to \
replace the following line with one that will work for their machine. Here is \
the complete path to the eighth individual in ",
  StyleBox["1C/",
    FontFamily->"Courier"],
  ", where the GECCOExamples directory is on my desktop for a Mac OS X \
machine."
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[
    \(\(nodFile\  = \ "\</Users/daida/Desktop/1C/tree8.nod\>";\)\)], "Input"],

Cell[TextData[{
  "Other individuals can be visualized by editing the above filepath.\n\n\
Visualizing this individual is otherwise similar to visualizing a simulated \
tree. The main difference is in reading in the data from a file, instead of \
generating it via ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[{
    \(\(data\  = \ 
        ReadList[nodFile, \ 
          RecordSeparators \[Rule] {"\<\n\>"}];\)\), "\[IndentingNewLine]", 
    \(\(maximumDepth\  = \ 26;\)\), "\[IndentingNewLine]", 
    \(\(circleGraph = 
        Graphics[{GrayLevel[0.5], 
            Circle[{0, 0}, maximumDepth]}];\)\), "\[IndentingNewLine]", 
    \(\(Show[GraphTreeTwo[data], \ circleGraph, \ AspectRatio \[Rule] 1, \ 
        PlotRange \[Rule] All];\)\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Visualization of a Population of GP Individuals", "Subsection"],

Cell[TextData[{
  "The code ",
  StyleBox["GrayScaleTree",
    FontFamily->"Courier"],
  " can generate a summary visualiation for an entire population. It is \
similar to the previously used code that generates a single tree in that the \
routine takes as input a set of labels. The format of the input variables, \
however, is slightly different to account for the cumulative frequencies that \
are tagged to each label. As a convenience, we have provided a routine ",
  StyleBox["SummarizePopulation",
    FontFamily->"Courier"],
  " that takes as input individual tree files and gives as output the \
formatted data that can be used by ",
  StyleBox["GrayScaleTree",
    FontFamily->"Courier"],
  ". \n\nHere is an example of using ",
  StyleBox["GrayScaleTree",
    FontFamily->"Courier"],
  " to generate a population visualization. To experiment, you will need to \
set the variable numberOfIndividuals, which is the total number of \
individuals in a population to be summarized, as well as the directory in \
which those individuals are found. For this particular example, only the \
first 20 individuals out of the 500 are indicated. (Although you can \
visualize the entire population of 500, it does take a while.)"
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[{
    \(\(numberOfIndividuals\  = \ 20;\)\), "\[IndentingNewLine]", 
    \(\(treeDir\  = \ "\</Users/daida/Desktop/1C/\>";\)\)}], "Input"],

Cell[TextData[{
  "The remaining code is similar to the visualization of individual trees, \
with the exception of the calling ",
  StyleBox["SummarizePopulation",
    FontFamily->"Courier"],
  "."
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[{
    \(\(unityNodes\  = \ 
        SummarizePopulation[treeDir, 
          numberOfIndividuals];\)\), "\[IndentingNewLine]", 
    \(\(maximumSpecifiedDepth\  = \ 26;\)\), "\[IndentingNewLine]", 
    \(\(circleGraph = 
        Graphics[{GrayLevel[0.5], 
            Circle[{0, 0}, 
              maximumSpecifiedDepth]}];\)\), "\[IndentingNewLine]", 
    \(\(unityPopTreeGraphic\  = 
        GrayscaleTree[unityNodes];\)\), "\[IndentingNewLine]", 
    \(\(oneGraphic\  = \ 
        Show[unityPopTreeGraphic, \ circleGraph, \ AspectRatio \[Rule] 1, 
          PlotRange \[Rule] All];\)\)}], "Input"],

Cell[TextData[{
  "It is possible to visualize a sequence of populations, as well. To do \
this, we've precomputed the population summaries from four different trials \
that were featured in the ",
  StyleBox["GPEM",
    FontSlant->"Italic"],
  " version of the visualization paper. These population summaries are found \
in directory ",
  StyleBox["GPEM/",
    FontFamily->"Courier"],
  ". We've included here an example from \[Alpha] = 1000, tournament \
selection. Although we provide population summaries enough for 100 \
generations, we only show the first 20. The visualization of all 100 \
generations can take a while..."
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[{
    \(\(populationDirectory\  = \ \
"\</Users/daida/Desktop/alpha1k_tournament/trial1/\>";\)\), "\
\[IndentingNewLine]", 
    \(\(numberOfGenerations = \ 20;\)\), "\[IndentingNewLine]", 
    \(\(maximumSpecifiedDepth\  = \ 26;\)\)}], "Input"],

Cell[TextData[{
  "The function ",
  StyleBox["VisualizeTrialGray",
    FontFamily->"Courier"],
  " is what converts population summaries to a sequence. As in all ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " animations, select the all of the graphics for animation, then go to \
menubar item \"Cell > Animate Selected Graphics\". Notice that the \
visualizations start at generation 1 and not generation 0."
}], "Text",
  PageWidth->PaperWidth],

Cell[BoxData[
    \(\(VisualizeTrialGray[populationDirectory, numberOfGenerations, \ 
        maximumSpecifiedDepth];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["About Examples", "Section"],

Cell[TextData[{
  "There are several data sets that accompany this notebook. They are as \
follows:\n\nFor use with ",
  StyleBox["GraphTreeTwo[] ",
    FontFamily->"Courier"],
  "and ",
  StyleBox["GraphTreeNodesTwo[]",
    FontFamily->"Courier"],
  ":\n\n1C/\t\tContains data for 500 individual trees. Proportionate \
selection,  generation 200, \[Alpha] = 1\n1kC/\t\tContains data for 500 \
individual trees. Proportionate selection,  generation 200, \[Alpha] = 1000\n\
\nFor use with ",
  StyleBox["GrayscaleTree[] ",
    FontFamily->"Courier"],
  "and ",
  StyleBox["VisualizeTrialGray[]",
    FontFamily->"Courier"],
  ":\n\nalpha1_tournament/\t\tContains data for 100 generations of 500 \
individuals. Tournament selection (",
  StyleBox["n",
    FontSlant->"Italic"],
  "=7),   \[Alpha] = 1\nalpha1_proportionate/\tContains data for 100 \
generations of 500 individuals. Proportionate selection,   \[Alpha] = 1\n\
alpha1k_tournament/\t\tContains data for 100 generations of 500 individuals. \
Tournament selection (",
  StyleBox["n",
    FontSlant->"Italic"],
  "=7),   \[Alpha] = 1000\nalpha1k_proportionate/\tContains data for 100 \
generations of 500 individuals. Proportionate selection,   \[Alpha] = 1000"
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Acknowledgments", "Section"],

Cell["\<\
We thank Sandra Larson and Jason Kriesel for editing and testing \
the code in this document.\
\>", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bibliography", "Section"],

Cell[TextData[{
  "J. M. Daida, A. Hilss, D. J. Ward, and S. Long, \
\[OpenCurlyDoubleQuote]Visualizing Tree Structures in Genetic Programming,\
\[CloseCurlyDoubleQuote] in ",
  StyleBox["Genetic and Evolutionary Computation\[LongDash]GECCO 2003: \
Genetic and Evolutionary Computation Conference, Chicago, IL, USA, July 2003",
    
    FontSlant->"Italic"],
  ", vol. LNCS 2724, Lecture Notes in Computer Science, E. Cant\[UAcute]-Paz, \
J. A. Foster, K. Deb, L. D. Davis, R. Roy, U.-M. O\[CloseCurlyQuote]Reilly, \
H.-G. Beyer, R. Standish, G. Kendall, S. Wilson, M. Harman, J. Wegener, D. \
Dasgupta, M. A. Potter, A. C. Schultz, K. A. Dowsland, and N. J. J. Miller, \
Eds. Berlin: Springer-Verlag, 2003, pp. 1652\[Dash]1664.\n\nJ. M. Daida, A. \
Hilss, D. J. Ward, and S. Long, \[OpenCurlyDoubleQuote]Visualizing Tree \
Structures in Genetic Programming,\[CloseCurlyDoubleQuote] ",
  StyleBox["Journal of G",
    FontSlant->"Italic"],
  StyleBox["enetic Programming and Evolutionary Machines",
    FontSlant->"Italic"],
  ", forthcoming.\n\n",
  StyleBox["J. M. Daida and A. Hilss, \[OpenCurlyDoubleQuote]Identifying \
Structural Mechanisms in Standard Genetic \
Programming,\[CloseCurlyDoubleQuote] in ",
    FontFamily->"Times New Roman"],
  StyleBox["Genetic and Evolutionary Computation\[LongDash]GECCO 2003: \
Genetic and Evolutionary Computation Conference, Chicago, IL, USA, July 2003",
    
    FontSlant->"Italic"],
  StyleBox[", vol. LNCS 2724, Lecture Notes in Computer Science, E. Cant\
\[UAcute]-Paz, J. A. Foster, K. Deb, L. D. Davis, R. Roy, U.-M. O\
\[CloseCurlyQuote]Reilly, H.-G. Beyer, R. Standish, G. Kendall, S. Wilson, M. \
Harman, J. Wegener, D. Dasgupta, M. A. Potter, A. C. Schultz, K. A. Dowsland, \
and N. J. J. Miller, Eds. Berlin: Springer-Verlag, 2003, pp. \
1639\[Dash]1651.",
    FontFamily->"Times New Roman"]
}], "Text",
  PageWidth->PaperWidth]
}, Open  ]],

Cell[CellGroupData[{

Cell["License", "Section"],

Cell[TextData[{
  "The software in the section entitled \[OpenCurlyDoubleQuote]Code\
\[CloseCurlyDoubleQuote] is copyrighted 2004 by Jason Daida and Adam Hilss. \
All rights reserved.\n\nThe software is provided as is.The authors make no \
guarantee about the code other than validating portions of the GECCO and GPEM \
papers that have evoked this code.You are free to use or modify any of this \
code",
  StyleBox[" so long as you attribute its use to this notebook",
    FontSlant->"Italic"],
  ":\n\nDaida, J.M., A.M. Hilss, D.J. Ward, and S.L. Long, ",
  StyleBox["Visualization of Tree Structures: Examples from Genetic \
Programming and a Model of Tree Growth",
    FontSlant->"Italic"],
  ", University of Michigan, Release 3 March 2004."
}], "Text",
  PageWidth->PaperWidth]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 723}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Working",
PrintingStyleEnvironment->"Printout",
WindowSize->{855, 696},
WindowMargins->{{Automatic, 32}, {Automatic, 0}},
Magnification->1,
StyleDefinitions -> "Default.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 192, 4, 89, "Title"],
Cell[1971, 59, 132, 2, 58, "Subtitle"],
Cell[2106, 63, 192, 4, 27, "Subsubtitle"],
Cell[2301, 69, 318, 5, 85, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2644, 78, 27, 0, 73, "Section"],
Cell[2674, 80, 2361, 45, 527, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5072, 130, 103, 2, 73, "Section",
  InitializationCell->True],
Cell[5178, 134, 426, 10, 104, "Text",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[5629, 148, 68, 1, 46, "Subsection",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[5722, 153, 159, 5, 42, "Subsubsection",
  InitializationCell->True],
Cell[5884, 160, 1427, 40, 582, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[7348, 205, 86, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[7437, 208, 3075, 98, 1452, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[10549, 311, 95, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[10647, 314, 682, 21, 297, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[11366, 340, 87, 1, 28, "Subsubsection",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[11478, 345, 1884, 55, 792, "Input",
  InitializationCell->True],
Cell[13365, 402, 192, 4, 70, "Message"],
Cell[13560, 408, 179, 3, 70, "Message"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[13788, 417, 101, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[13892, 420, 1238, 35, 507, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[15167, 460, 93, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[15263, 463, 2315, 79, 1167, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[17615, 547, 87, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[17705, 550, 643, 18, 252, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[18385, 573, 87, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[18475, 576, 226, 9, 117, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[18738, 590, 139, 4, 28, "Subsubsection",
  InitializationCell->True],
Cell[18880, 596, 224, 6, 50, "Text",
  InitializationCell->True],
Cell[19107, 604, 4496, 76, 1083, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[23640, 685, 138, 4, 28, "Subsubsection",
  InitializationCell->True],
Cell[23781, 691, 3240, 53, 619, "Input",
  InitializationCell->True],
Cell[27024, 746, 4056, 76, 971, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[31117, 827, 85, 1, 28, "Subsubsection"],
Cell[31205, 830, 254, 5, 50, "Text"],
Cell[31462, 837, 4711, 77, 875, "Input",
  InitializationCell->True]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[36222, 920, 65, 1, 30, "Subsection",
  InitializationCell->True],
Cell[36290, 923, 749, 12, 122, "Text"],

Cell[CellGroupData[{
Cell[37064, 939, 95, 1, 42, "Subsubsection",
  InitializationCell->True],
Cell[37162, 942, 2130, 70, 1032, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[39329, 1017, 100, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[39432, 1020, 2798, 87, 1287, "Input",
  InitializationCell->True]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{
Cell[42279, 1113, 55, 1, 30, "Subsection",
  InitializationCell->True],

Cell[CellGroupData[{
Cell[42359, 1118, 72, 1, 42, "Subsubsection",
  InitializationCell->True],
Cell[42434, 1121, 110, 4, 42, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[42581, 1130, 86, 1, 28, "Subsubsection",
  InitializationCell->True],
Cell[42670, 1133, 218, 9, 117, "Input",
  InitializationCell->True]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[42949, 1149, 56, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[43030, 1153, 67, 0, 38, "Subsection"],
Cell[43100, 1155, 416, 9, 90, "Text"],
Cell[43519, 1166, 273, 6, 71, "Text"],
Cell[43795, 1174, 55, 1, 30, "Input"],
Cell[43853, 1177, 244, 7, 52, "Text"],
Cell[44100, 1186, 90, 1, 30, "Input"],
Cell[44193, 1189, 107, 3, 33, "Text"],
Cell[44303, 1194, 215, 3, 70, "Input"],
Cell[44521, 1199, 614, 14, 109, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[45172, 1218, 72, 0, 38, "Subsection"],
Cell[45247, 1220, 481, 9, 109, "Text"],
Cell[45731, 1231, 117, 2, 50, "Input"],
Cell[45851, 1235, 53, 0, 33, "Text"],
Cell[45907, 1237, 69, 0, 30, "Input"],
Cell[45979, 1239, 226, 5, 52, "Text"],
Cell[46208, 1246, 224, 5, 84, "Input"],
Cell[46435, 1253, 160, 4, 52, "Text"],
Cell[46598, 1259, 504, 10, 110, "Input"],
Cell[47105, 1271, 283, 6, 71, "Text"],
Cell[47391, 1279, 84, 1, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47512, 1285, 60, 0, 38, "Subsection"],
Cell[47575, 1287, 894, 21, 166, "Text"],
Cell[48472, 1310, 180, 3, 70, "Input"],
Cell[48655, 1315, 114, 2, 30, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[48806, 1322, 67, 0, 38, "Subsection"],
Cell[48876, 1324, 2208, 41, 508, "Text"],
Cell[51087, 1367, 91, 1, 30, "Input"],
Cell[51181, 1370, 362, 9, 90, "Text"],
Cell[51546, 1381, 455, 9, 90, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52038, 1395, 69, 0, 38, "Subsection"],
Cell[52110, 1397, 1264, 25, 280, "Text"],
Cell[53377, 1424, 151, 2, 50, "Input"],
Cell[53531, 1428, 234, 7, 52, "Text"],
Cell[53768, 1437, 611, 13, 110, "Input"],
Cell[54382, 1452, 666, 15, 128, "Text"],
Cell[55051, 1469, 257, 5, 70, "Input"],
Cell[55311, 1476, 458, 11, 90, "Text"],
Cell[55772, 1489, 132, 2, 30, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[55953, 1497, 33, 0, 73, "Section"],
Cell[55989, 1499, 1230, 29, 261, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57256, 1533, 34, 0, 73, "Section"],
Cell[57293, 1535, 117, 3, 33, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[57447, 1543, 31, 0, 73, "Section"],
Cell[57481, 1545, 1883, 36, 356, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[59401, 1586, 26, 0, 73, "Section"],
Cell[59430, 1588, 783, 15, 204, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

